from typing import Any, final

from cow.errors.exception import VLException
from cow.pydantic.loaders import loadDataFromJson
from cow.web.request import VLRequest
from luna3.client import Client as _Client
from sanic import Sanic
from vlutils.structures.pydantic import BaseModel

from .base_application import BaseAppCtx
from .exceptions import BaseLambdaException
from .monitoring import MonitoringPoint


class BaseLambdaRequest(VLRequest[BaseAppCtx]):
    """
    Luna-configurator request
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.accountId = self.headers.get("Luna-Account-Id")
        self.userCtx = self.app.ctx.userCtx

    @property
    def json(self) -> int | float | list | dict[str, Any]:
        """Get json from request"""
        return super().json

    def getBody(self) -> bytes:
        """Get body from request"""
        return super().body

    def sendToMonitoring(self, points: list[MonitoringPoint]) -> None:
        """
        Send point to monitoring

        Args:
            points: points
        """
        if self.app.ctx.serviceConfig.monitoring.sendData:
            self.app.ctx.monitoring.flushPoints(points)


class ClientsKwargsMixin:
    """Luna services clients kwargs mixin"""

    _luna3Client: _Client
    accountId: str
    app: Sanic

    def _getFacesClientKwargs(self) -> dict:
        """Get faces clients kwargs"""
        return {
            "facesClient": self.appCtx.luna3session.facesSession.getClient(self.requestId),
            "accountId": self.accountId,
        }

    def _getEventsClientKwargs(self) -> dict:
        """Get events clients kwargs"""
        return {
            "eventsClient": self.appCtx.luna3session.eventSession.getClient(self.requestId),
            "accountId": self.accountId,
        }

    def _getLPMClientKwargs(self) -> dict:
        """Get lpn clients kwargs"""
        return {
            "lpmClient": self.appCtx.luna3session.pythonMatcherSession.getClient(self.requestId),
            "accountId": self.accountId,
        }

    def _getLVAClientKwargs(self) -> dict:
        """Get lpn clients kwargs"""
        return {
            "lvaClient": self.appCtx.luna3session.videoAgentSession.getClient(self.requestId),
            "accountId": self.accountId,
        }

    def _getFacesSamplesClientKwargs(self) -> dict:
        """Get faces samples clients kwargs"""
        return {
            "lisClient": self.appCtx.luna3session.faceSampleStoreSession.getClient(self.requestId),
            "accountId": self.accountId,
            "bucket": self.app.ctx.serviceConfig.faceSamplesStorage.bucket,
        }

    def _getBodySamplesClientKwargs(self) -> dict:
        """Get body samples clients kwargs"""
        return {
            "lisClient": self.appCtx.luna3session.bodySampleStoreSession.getClient(self.requestId),
            "accountId": self.accountId,
            "bucket": self.app.ctx.serviceConfig.bodySamplesStorage.bucket,
        }

    def _getImageOriginClientKwargs(self) -> dict:
        """Get image origin clients kwargs"""
        return {
            "lisClient": self.appCtx.luna3session.imageOriginStoreSession.getClient(self.requestId),
            "accountId": self.accountId,
            "bucket": self.app.ctx.serviceConfig.imageOriginStorage.bucket,
        }

    def _getLRSClientKwargs(self) -> dict:
        """Get lrs clients kwargs"""
        return {
            "lrsClient": self.appCtx.luna3session.remoteSDKSession.getClient(self.requestId),
            "accountId": self.accountId,
        }

    def _getSenderClientKwargs(self) -> dict:
        """Get sender clients kwargs"""
        return {
            "senderClient": self.appCtx.luna3session.senderSession.getClient(self.requestId),
            "accountId": self.accountId,
        }

    def _getHandlersClientKwargs(self) -> dict:
        """Get handlers clients kwargs"""
        return {
            "handlersClient": self.appCtx.luna3session.handlersSession.getClient(self.requestId),
            "accountId": self.accountId,
        }


class BaseStandaloneLambdaRequest(BaseLambdaRequest, ClientsKwargsMixin):
    """Base lambda request"""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._luna3Client: _Client = self.app.ctx.luna3session.getClient(self.requestId)


class BaseHandlersRequest(BaseStandaloneLambdaRequest):
    """Base lambda request"""

    @final
    def getRequestData(self, pydanticModel: type[BaseModel]):
        """Get data from request"""
        try:
            return loadDataFromJson(super().json, pydanticModel)
        except VLException as e:
            exc = BaseLambdaException
            exc.errorText = str(e.error.asDict())
            exc.statusCode = 400
            raise exc
